#include "CKlearControllerTXTMain.h"

CKlearControllerTXTMain::CKlearControllerTXTMain( CKlearAppConfig *config, QWidget* parent, const char* name, WFlags fl ) : CKlearUITXTMain( parent, name, fl )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::CKlearControllerTXTMain()";

  this->digit = 0;
  this->asci = "";
  this->pageNumber = "100";
  this->doubleHeight = 1;
  this->counter = 0;
  this->row = 0;

  this->KlearConfig = config;
  this->TXTPainter = new QPainter( this );
  this->TXTDecoder = new CKlearAppTXTDecoder( this->KlearConfig );
  this->TXTStreamer = new CKlearAppTXTStreamer( this->KlearConfig, this->TXTDecoder );

  createTXTPainter( this->TXTPainter );
  this->TXTStreamer->start();
  this->TXTDecoder->pageNumber = this->pageNumber;

  connect( this->TXTDecoder, SIGNAL( changedHeaderInfo() ), this, SLOT( slotGrabSavedHeader() ) );
  connect( this->TXTDecoder, SIGNAL( changedBodyInfo() ), this, SLOT( slotGrabSavedBody() ) );
}

CKlearControllerTXTMain::~CKlearControllerTXTMain()
{
  const QString __FUNC__ = "CKlearControllerTXTMain::~CKlearControllerTXTMain()";

  delete this->TXTPainter;
  delete this->TXTDecoder;
  delete this->TXTStreamer;
  delete this->KlearConfig;
}

void CKlearControllerTXTMain::paintEvent( QPaintEvent *p )
{
    const QString __FUNC__ = "CKlearControllerTXTMain::paintEvent( QPaintEvent *p )";

    if( this->pageNumber != "100" )
    {
      paintTXTPageNumber( this->TXTPainter );
      paintTXTHeader( this->TXTPainter );
      paintTXTBody( this->TXTPainter );
    }
}

void CKlearControllerTXTMain::createTXTPainter( QPainter *p )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::createTXTPainter( QPainter *p )";

  this->setCaption( "Klear - Teletext: " + this->KlearConfig->getCurrentChannel() );
  this->setFixedSize( 680, 525 );

  this->rectWidth = ( float )this->width() / 40;
  this->rectHeight = ( float )this->height() / 25;

  getTXTFont();
}

QString CKlearControllerTXTMain::getPageNumber()
{
  const QString __FUNC__ = "CKlearControllerTXTMain::getPageNumber()";

  return this->pageNumber;
}

void CKlearControllerTXTMain::slotGrabSavedHeader()
{
  const QString __FUNC__ = "CKlearControllerTXTMain::slotGrabSavedHeader()";

  paintTXTHeader( this->TXTPainter );
}

void CKlearControllerTXTMain::slotGrabSavedBody()
{
  const QString __FUNC__ = "CKlearControllerTXTMain::slotGrabSavedBody()";

  this->counter = 3;
  paintTXTBody( this->TXTPainter );
}

void CKlearControllerTXTMain::paintTXTHeader(  QPainter *p  )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::paintTXTHeader( QPainter *p )";

  setDefaultProperties();

  for( int i = counter; i < 32; i++ )
  {
    setCharakterColor( this->TXTDecoder->TXTHeaderData[ i ] );
    setMosaicColor( this->TXTDecoder->TXTHeaderData[ i ] );
    setLatinNationalOption( this->TXTDecoder->TXTHeaderData[ i ] );

    p->fillRect( getRectWidth() * ( i + 8 ), 0, getRectWidth(), getRectHeight(),  getBackgroundColor() );
    p->setPen( getCharakterColor() );
    p->setFont( getTXTFont() );
    p->drawText( getRectWidth() * ( i + 8 ), 0, getRectWidth(), getRectHeight(), Qt::AlignCenter, this->asci );
  }
}

void CKlearControllerTXTMain::paintTXTBody ( QPainter *p  )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::paintTXTBody( QPainter *p )";

  setDefaultProperties();

  for( int column = 0; column < 3; column++ )
  {
    this->asci = this->pageNumber.at( column );

    p->fillRect( getRectWidth() * ( column + 8 ), 0, getRectWidth(), getRectHeight(),  getBackgroundColor() );
    p->setPen( getCharakterColor() );
    p->setFont( getTXTFont() );
    p->drawText( getRectWidth() * ( column + 8 ), 0, getRectWidth(), getRectHeight(), Qt::AlignCenter, this->asci );
  }

  this->position = 0;
  this->row = 0;

  while( this->position != 24 )
  {
    setDefaultProperties();
    setDoubleHeight( this->row );

    for ( int column = 0; column < 40; column++ )
    {
      if( this->TXTDecoder->TXTBodyData[ this->row ][ column ] >= 0 && this->TXTDecoder->TXTBodyData[ this->row ][ column ] <= 7 )
        setCharakterColor( this->TXTDecoder->TXTBodyData[ this->row ][ column ] );

      if( this->TXTDecoder->TXTBodyData[ this->row ][ column ] >= 16 && this->TXTDecoder->TXTBodyData[ this->row ][ column ] <= 23 )
        setMosaicColor( this->TXTDecoder->TXTBodyData[ this->row ][ column ] );

      if( this->TXTDecoder->TXTBodyData[ this->row ][ column ] == 29 )
        setBackgroundColor( this->TXTDecoder->TXTBodyData[ this->row ][ column ] );

      if( this->element == "char")
      {
        setLatinNationalOption( this->TXTDecoder->TXTBodyData[ this->row ][ column ] );
        p->fillRect( getRectWidth() * column, getRectHeight() * ( this->position + 1 ), getRectWidth(), getRectHeight() * this->doubleHeight, getBackgroundColor() );
        p->setPen( getCharakterColor() );
        p->setFont( getTXTFont() );
        p->drawText( getRectWidth() * column, getRectHeight() * ( this->position + 1 ), getRectWidth(), getRectHeight() * this->doubleHeight, Qt::AlignCenter, this->asci );
      }

      if( this->element == "mosaic" && this->TXTDecoder->TXTBodyData[ this->row ][ column ] > 23 )
      {
        p->fillRect( getRectWidth() * column, getRectHeight() * ( this->position + 1 ), getRectWidth(), getRectHeight() * this->doubleHeight, getBackgroundColor() );

        setBinary( this->TXTDecoder->TXTBodyData[ this->row ][ column ] );

        int pixelWidth = getRectWidth() / 2;
        int pixelHeight = getRectHeight() / 3;

        if( this->binary[ 0 ] == 1 )
          p->fillRect( getRectWidth() * column + pixelWidth, getRectHeight() * ( this->position + 1 ) + ( 3 * pixelHeight ), pixelWidth, pixelHeight,      getMosaicColor() );

        if( this->binary[ 2 ] == 1 )
          p->fillRect( getRectWidth() * column, getRectHeight() * ( this->position + 1 ) + ( 3 * pixelHeight ), pixelWidth, pixelHeight,      getMosaicColor() );

        if( this->binary[ 3 ] == 1 )
          p->fillRect( getRectWidth() * column + pixelWidth, getRectHeight() * ( this->position + 1 ) + ( 2 * pixelHeight ), pixelWidth, pixelHeight,      getMosaicColor() );

        if( this->binary[ 4 ] == 1 )
          p->fillRect( getRectWidth() * column, getRectHeight() * ( this->position + 1 ) + ( 2 * pixelHeight ), pixelWidth, pixelHeight,      getMosaicColor() );

        if( this->binary[ 5 ] == 1 )
          p->fillRect( getRectWidth() * column + pixelWidth, getRectHeight() * ( this->position + 1 ) +  pixelHeight, pixelWidth, pixelHeight,      getMosaicColor() );

        if( this->binary[ 6 ] == 1 )
          p->fillRect( getRectWidth() * column, getRectHeight() * ( this->position + 1 ) +  pixelHeight, pixelWidth, pixelHeight, getMosaicColor() );
      }
    }

    this->position++;
    this->row++;

    if ( this->doubleHeight == 2 )
    {
      this->doubleHeight = 1;
      this->position++;
    }
  }
}

void CKlearControllerTXTMain::setBinary( int dezimal )
{
  for ( int i = 6; i >= 0; i-- )
  {
    this->binary[ i ] = dezimal % 2;
    dezimal = dezimal / 2;
  }
  std::cout << "Binary: ";
  for ( int i = 0; i < 7; i++ )
    std::cout << binary[ i ];
  std::cout << std::endl;
}

void CKlearControllerTXTMain::keyPressEvent( QKeyEvent *ke )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::keyPressEvent( QKeyEvent *ke )";

  QString num;
  for ( int i = 0; i <= 9; i++ )
  {
    num = num.setNum( i );
    if( ke->text() == num )
      setTXTPageNumber( num );
  }
}

void CKlearControllerTXTMain::setTXTPageNumber( QString num )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::setTXTNumber( QString num, QPainter *p )";

  this->pageNumber = "";

  if( this->digit == 0 )
  {
    this->number[ 0 ] = num;
    this->number[ 1 ] = "-";
    this->number[ 2 ] = "-";
  }

  if ( this->digit == 1 )
  {
    this->number[ 1 ] = num;
    this->number[ 2 ] = "-";
  }

  if ( this->digit == 2 )
    this->number[ 2 ] = num;

  for ( int i = 0; i < 3; i++ )
    this->pageNumber += this->number[ i ];

  paintTXTPageNumber( this->TXTPainter );

  this->digit++;
  if( this->digit == 3 )
  {
    std::cout << "Page Number: " << this->pageNumber << std::endl;
    this->TXTDecoder->pageNumber = this->pageNumber;
    this->digit = 0;
    this->counter = 0;
  }
}

void CKlearControllerTXTMain::closeEvent( QCloseEvent *ce )
{
  const QString __FUNC__ = "CKlearControllerTXTMain::closeEvent( QCloseEvent *ce )";

  this->TXTStreamer->setRunning( false );

  while( this->TXTStreamer->running() )
    this->TXTStreamer->wait();
  std::cout << "TXT stopped" << std::endl;

  this->hide();
  this->close();
}


void CKlearControllerTXTMain::setCharakterColor( int hex )
{
  switch( hex )
  {
    case 0: this->charakterColor = Qt::black; break;
    case 1: this->charakterColor = Qt::red; break;
    case 2: this->charakterColor = Qt::green; break;
    case 3: this->charakterColor = Qt::yellow; break;
    case 4: this->charakterColor = Qt::blue; break;
    case 5: this->charakterColor = Qt::magenta; break;
    case 6: this->charakterColor = Qt::cyan; break;
    case 7: this->charakterColor = Qt::white; break;
  }
  this->element = "char";
}

void CKlearControllerTXTMain::setMosaicColor( int hex )
{
  switch( hex )
  {
    case 16: this->mosaicColor = Qt::black; break;
    case 17: this->mosaicColor = Qt::red; break;
    case 18: this->mosaicColor = Qt::green; break;
    case 19: this->mosaicColor = Qt::yellow; break;
    case 20: this->mosaicColor = Qt::blue; break;
    case 21: this->mosaicColor = Qt::magenta; break;
    case 22: this->mosaicColor = Qt::cyan; break;
    case 23: this->mosaicColor = Qt::white; break;
  }
  this->element = "mosaic";
}

void CKlearControllerTXTMain::setBackgroundColor( int hex )
{
  this->backgroundColor = this->charakterColor;
}

void CKlearControllerTXTMain::setLatinNationalOption( int nationalOption )
{
  switch( nationalOption )
  {
    case 35: this->asci = "#"; break;
    case 36: this->asci = "$"; break;
    case 64: this->asci = ""; break;
    case 91: this->asci = ""; break;
    case 92: this->asci = ""; break;
    case 93: this->asci = ""; break;
    case 94: this->asci = "^"; break;
    case 95: this->asci = "_"; break;
    case 96: this->asci = ""; break;
    case 123: this->asci = ""; break;
    case 124: this->asci = ""; break;
    case 125: this->asci = ""; break;
    case 126: this->asci = ""; break;
    default: this->asci = nationalOption;
  }
}

void CKlearControllerTXTMain::setDoubleHeight( int row )
{
  for ( int i = 0; i < 40; i++ )
    if ( this->TXTDecoder->TXTBodyData[ row ][ i ] == 13 )
      this->doubleHeight = 2;
}

QColor CKlearControllerTXTMain::getCharakterColor()
{
  return this->charakterColor;
}

QColor CKlearControllerTXTMain::getMosaicColor()
{
  return this->mosaicColor;
}

QColor CKlearControllerTXTMain::getBackgroundColor()
{
  return this->backgroundColor;
}

float CKlearControllerTXTMain::getRectHeight()
{
  return this->rectHeight;
}

float CKlearControllerTXTMain::getRectWidth()
{
  return this->rectWidth;
}

void CKlearControllerTXTMain::setDefaultProperties()
{
  this->backgroundColor = Qt::black;
  this->charakterColor = Qt::white;
  this->doubleHeight = 1;
}

void CKlearControllerTXTMain::paintTXTPageNumber( QPainter *p )
{
  for ( int i = 0; i < 2; i++ )
    p->fillRect( getRectWidth() * i , 0, getRectWidth(), getRectHeight(), Qt::black );

  for ( int i = 0; i < 3; i++ )
  {
    p->fillRect( getRectWidth() * ( i + 2 ), 0, getRectWidth(), getRectHeight(), Qt::black );
    p->setPen( Qt::white );
    p->setFont( getTXTFont() );
    p->drawText( getRectWidth() * ( i + 2 ), 0, getRectWidth(), getRectHeight(), Qt::AlignLeft, this->number[ i ] );
  }

  for ( int i = 0; i < 2; i++ )
    p->fillRect( getRectWidth() * ( i + 5 ), 0, getRectWidth(), getRectHeight(), Qt::black );
}

QFont CKlearControllerTXTMain::getTXTFont()
{
  TXTFont.setFamily( "Helvetica" );
  TXTFont.setBold( true );
  TXTFont.setPixelSize( getRectHeight() - 1 );
  return TXTFont;
}